<?php

#############################################################################
#
# Utility functions.
#
#############################################################################

function service_colour($key) {
  global $services;

  if (ereg ("ALL",$key)) return "<FONT COLOR=\"#FF0000\">";
  if (ereg ("HIPORTS",$key)) return "<FONT COLOR=\"#FF0000\">";

  $port = $services[$key];
  $exploded = preg_split ("/[\s\/]+/", $key);
  $proto = $exploded[1];

  if ($port < 1024) {
    if ($proto == "tcp") {
      $fs = "<FONT COLOR=\"#CC00CC\">";
    } elseif ($proto == "udp") {
      $fs = "<FONT COLOR=\"#EE0000\">";
    } else {
      $fs = "<FONT COLOR=\"#0000CC\">";
    }
  } else {
    if ($proto == "tcp") {
      $fs = "<FONT COLOR=\"#777700\">";
    } elseif ($proto == "udp") {
      $fs = "<FONT COLOR=\"#007777\">";
    } else {
      $fs = "<FONT COLOR=\"#0000EE\">";
    }
  }
  return $fs;
}

function option_colour($string) {
  if (ereg ("YES",$string)) return "<FONT COLOR=\"#00FF00\">";
  if (ereg ("ACCEPT",$string)) return "<FONT COLOR=\"#00FF00\">";
  if (ereg ("NO",$string)) return "<FONT COLOR=\"#FF0000\">";
  if (ereg ("DENY",$string)) return "<FONT COLOR=\"#FF0000\">";
}

#############################################################################
#
# Generate the firewall script.
#
#############################################################################


function gen_script() {

  global $CONF;

  $protocols = read_protocols();
  $services = read_services();
  $netobjects = read_objects();
  $rules = read_rulesets();
  $portfwds = read_portfwds();
  $redirectors = read_redirectors();

  $ichn = 1;
  $fchn = 1;
  $ochn = 1;
  $outrules = 0;

  #
  # Generate the file.
  #
  $fwf = fopen("$CONF/firewall", "w+");
  chmod( "$CONF/firewall", 0755 );

  fwrite ($fwf, "#!/bin/sh\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Firewall script generated by the PHP Firewall Generator (phpfwgen).\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# PHP Firewall Generator by Del (del@babel.com.au).  Original coding ideas\n");
  fwrite ($fwf, "# from IPCHAINS-FIREWALL V1.6.2m by Ian Hall-Beyer\n");
  fwrite ($fwf, "# and various others.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# This script is automatically generated.  Edit at your own peril.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# phpfwgen (C) Del under the GPL.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# This script is _not_ copyright to anyone, since it was generated by\n");
  fwrite ($fwf, "# software.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "# Initialise Firewall Configuration\n");
  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "IPCHAINS='/sbin/ipchains'\n");
  fwrite ($fwf, "IPMASQADM='/usr/sbin/ipmasqadm'\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Network object definitions.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "PRIVPORTS='0:1023'\n");
  fwrite ($fwf, "LOCALPORTS=`sed 's/\W/:/' < /proc/sys/net/ipv4/ip_local_port_range`\n");
  fwrite ($fwf, "HIPORTS='1024:'\n");
  fwrite ($fwf, "\n");

  if (is_array($netobjects)) {
    reset($netobjects);
    while ( list( $key, $val ) = each( $netobjects ) ) {
      $exploded = explode (" ", $val);
      fwrite ($fwf, "$key='$exploded[0]/$exploded[1]'\n");
    }
  }
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Turn on forwarding and dynamic addressing.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "echo 1 > /proc/sys/net/ipv4/ip_forward\n");
  fwrite ($fwf, "echo 1 > /proc/sys/net/ipv4/ip_always_defrag\n");
  fwrite ($fwf, "echo 1 > /proc/sys/net/ipv4/ip_dynaddr\n");
  fwrite ($fwf, "echo 1 > /proc/sys/net/ipv4/tcp_syncookies\n");
  fwrite ($fwf, "echo 1 > /proc/sys/net/ipv4/icmp_ignore_bogus_error_responses\n");
  fwrite ($fwf, "echo 1 > /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Turn off ICMP redirects.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "for file in /proc/sys/net/ipv4/conf/*/accept_redirects; do\n");
  fwrite ($fwf, "  echo 0 > \$file\n");
  fwrite ($fwf, "done\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Turn off source routed packets.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "for file in /proc/sys/net/ipv4/conf/*/accept_source_route; do\n");
  fwrite ($fwf, "  echo 0 > \$file\n");
  fwrite ($fwf, "done\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Turn on anti-spoofing.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "for file in /proc/sys/net/ipv4/conf/*/rp_filter; do\n");
  fwrite ($fwf, "  echo 1 > \$file\n");
  fwrite ($fwf, "done\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Set IP Masquerade timeouts to sensible defaults.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "\$IPCHAINS -M -S 7200 10 60\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Flush all rulesets.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "\$IPCHAINS -F\n");
  fwrite ($fwf, "\$IPCHAINS -X\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Allow loopback interface\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# (#" . $ichn++ . ")\n");
  fwrite ($fwf, "\$IPCHAINS -A input -i lo -j ACCEPT\n");
  fwrite ($fwf, "# (#" . $ochn++ . ")\n");
  fwrite ($fwf, "\$IPCHAINS -A output -i lo -j ACCEPT\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "# Begin Rulesets\n");
  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "# Forwarding / Masquerading\n");
  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Turn off masquerading within masqueraded networks.\n");
  fwrite ($fwf, "#\n");

  if (is_array($netobjects)) {
    $netobjects_copy = $netobjects;
    reset($netobjects);
    while ( list($key,$val) = each($netobjects) ) {
      $exploded = explode (" ", $val);
      if ($exploded[2] == "YES") {
        #
        # Turn off masquerading between masq'd networks as well ...
        #
        reset($netobjects_copy);
        while ( list($keyc,$valc) = each($netobjects_copy) ) {
          $explodedc = explode (" ", $valc);
          if ($explodedc[2] == "YES") {
            fwrite ($fwf, "# (#" . $fchn++ . ")\n");
            fwrite ($fwf, "\$IPCHAINS -A forward -s \$$key -d \$$keyc -j ACCEPT\n");
          }
        }
      }
    }
  }

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Turn off masquerading of the external interface.\n");
  fwrite ($fwf, "#\n");
  if (isset ($netobjects["GATEWAYNET"])) {
    fwrite ($fwf, "# (#" . $fchn++ . ")\n");
    fwrite ($fwf, "\$IPCHAINS -A forward -s \$GATEWAYNET -d \$ALL -j ACCEPT\n");
    fwrite ($fwf, "\n");
  }

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Masquerade all other networks.\n");
  fwrite ($fwf, "#\n");

  if (is_array($netobjects)) {
    reset($netobjects);
    while ( list($key,$val) = each($netobjects) ) {
      $exploded = explode (" ", $val);
      if ($exploded[2] == "YES") {
        fwrite ($fwf, "# (#" . $fchn++ . ")\n");
        fwrite ($fwf, "\$IPCHAINS -A forward -s \$$key -d \$ALL -j MASQ\n");
      }
    }
  }
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Port Forwarding.\n");
  fwrite ($fwf, "#\n");

  if (is_array($portfwds)) {
    reset($portfwds);
    while ( list($key,$rule) = each($portfwds) ) {
      $rule = chop($rule);
      fwrite ($fwf, "#\n# Rule number $key [ $rule ]\n#\n");
      list ($proto, $source, $sport, $dest, $dport) = preg_split ("/[\s]+/", $rule);
      fwrite ($fwf, "\$IPMASQADM portfw -a -P $proto");
      list ($saddr, $mask, $masq) = preg_split ("/[\s]+/", $netobjects[$source]);
      list ($daddr, $mask, $masq) = preg_split ("/[\s]+/", $netobjects[$dest]);
      fwrite ($fwf, " -L $saddr $sport -R $daddr $dport\n");
    }
  }
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Redirectors.\n");
  fwrite ($fwf, "#\n");

  if (is_array($redirectors)) {
    reset($redirectors);
    while ( list($key,$rule) = each($redirectors) ) {
      $rule = chop($rule);
      fwrite ($fwf, "#\n# Rule number $key [ $rule ]\n#\n");
      list ($source, $dest, $proto, $sport, $dport) = preg_split ("/[\s]+/", $rule);
      fwrite ($fwf, "# (#" . $ichn++ . ")\n");
      fwrite ($fwf, "\$IPCHAINS -A input -p $proto");
      fwrite ($fwf, " -s \$$source -d \$$dest $sport -j REDIRECT $dport\n");
    }
  }
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "# Input/Forward/Output Firewall Rules\n");
  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "\n");

  if (is_array($rules)) {
    reset($rules);
    while ( list($key,$rule) = each($rules) ) {
      $rule = chop($rule);
      list ($source, $dest, $proto, $port, $action, $log, $direction)
        = preg_split ("/[\s]+/", $rule);
      # Do this for compatibility with previous versions of phpfwgen which
      # didn't use the direction flag ...
      if (empty($direction)) {
        $direction = "IN/FWD";
        $rule = "$rule $direction";
      }

      fwrite ($fwf, "#\n# Rule number $key [ $rule ]\n#\n");
      #
      # Yes, I know this is ugly ... the logic goes like this:  IN rules
      # should be added to the input chain only.  OUT rules go to the
      # output chain.  FWD rules go to the forward chain as well as the
      # output chain (in case there are output rules later that block
      # the output of the forwarded packet).  IN/FWD rules have to go
      # to all three chains.
      #
      switch ($direction) {
        case "IN":
          fwrite ($fwf, "# (#" . $ichn++ . ")\n");
          fwrite ($fwf, "\$IPCHAINS -A input");
        break;

        case "OUT":
          fwrite ($fwf, "# (#" . $ochn++ . ")\n");
          fwrite ($fwf, "\$IPCHAINS -A output");
          $outrules = 1;
        break;

        case "FWD":
          fwrite ($fwf, "# (#" . $fchn++ . ")\n");
          fwrite ($fwf, "\$IPCHAINS -A forward");
          if ($proto != "ALL") fwrite ($fwf, " -p $proto");
          if ($proto == "icmp") {
            fwrite ($fwf, " -s \$$source");
            if ($port != "ALL") fwrite ($fwf, " $port");
            fwrite ($fwf, " -d \$$dest");
          } else {
            fwrite ($fwf, " -s \$$source -d \$$dest");
            if ($port != "ALL") fwrite ($fwf, " $port");
          }
          fwrite ($fwf, " -j $action");
          if ($log == "YES") fwrite ($fwf, " -l");
          fwrite ($fwf, "\n");

          fwrite ($fwf, "# (#" . $ochn++ . ")\n");
          fwrite ($fwf, "\$IPCHAINS -A output");
        break;

        case "IN/FWD":
          fwrite ($fwf, "# (#" . $fchn++ . ")\n");
          fwrite ($fwf, "\$IPCHAINS -A forward");
          if ($proto != "ALL") fwrite ($fwf, " -p $proto");
          if ($proto == "icmp") {
            fwrite ($fwf, " -s \$$source");
            if ($port != "ALL") fwrite ($fwf, " $port");
            fwrite ($fwf, " -d \$$dest");
          } else {
            fwrite ($fwf, " -s \$$source -d \$$dest");
            if ($port != "ALL") fwrite ($fwf, " $port");
          }
          fwrite ($fwf, " -j $action");
          if ($log == "YES") fwrite ($fwf, " -l");
          fwrite ($fwf, "\n");

          fwrite ($fwf, "# (#" . $ichn++ . ")\n");
          fwrite ($fwf, "\$IPCHAINS -A input");
          if ($proto != "ALL") fwrite ($fwf, " -p $proto");
          if ($proto == "icmp") {
            fwrite ($fwf, " -s \$$source");
            if ($port != "ALL") fwrite ($fwf, " $port");
            fwrite ($fwf, " -d \$$dest");
          } else {
            fwrite ($fwf, " -s \$$source -d \$$dest");
            if ($port != "ALL") fwrite ($fwf, " $port");
          }
          fwrite ($fwf, " -j $action");
          if ($log == "YES") fwrite ($fwf, " -l");
          fwrite ($fwf, "\n");

          fwrite ($fwf, "# (#" . $ochn++ . ")\n");
          fwrite ($fwf, "\$IPCHAINS -A output");
        break;

      }

      if ($proto != "ALL") fwrite ($fwf, " -p $proto");
      if ($proto == "icmp") {
        fwrite ($fwf, " -s \$$source");
        if ($port != "ALL") fwrite ($fwf, " $port");
        fwrite ($fwf, " -d \$$dest");
      } else {
        fwrite ($fwf, " -s \$$source -d \$$dest");
        if ($port != "ALL") fwrite ($fwf, " $port");
      }
      fwrite ($fwf, " -j $action");
      if ($log == "YES") fwrite ($fwf, " -l");
      fwrite ($fwf, "\n");

      #
      # insert a reverse rule for ACCEPT (tcp) packets.
      #
      if (($action == "ACCEPT") and (($proto == "tcp") or ($proto == "ALL"))) {

        # This is even more ugly.  OUT packets require a return rule in the
        # input chain.  IN packets require a return input and output rule.
        # FWD and IN/FWD packets require a return forward, input, and output
        # rule.
        switch ($direction) {
          case "IN":
            fwrite ($fwf, "# (#" . $ichn++ . ") Allow return input packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A input");

            fwrite ($fwf, " -p tcp");
            fwrite ($fwf, " -d \$$source \$HIPORTS -s \$$dest");
            if ($port != "ALL") fwrite ($fwf, " $port");
            fwrite ($fwf, " ! -y -j $action");
            if ($log == "YES") fwrite ($fwf, " -l");
            fwrite ($fwf, "\n");

            fwrite ($fwf, "# (#" . $ochn++ . ") Allow return output packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A output");
          break;

          case "OUT":
            fwrite ($fwf, "# (#" . $ichn++ . ") Allow return input packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A input");
          break;

          case "FWD":
          case "IN/FWD":
            fwrite ($fwf, "# (#" . $fchn++ . ") Allow return fwd packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A forward");

            fwrite ($fwf, " -p tcp");
            fwrite ($fwf, " -d \$$source \$HIPORTS -s \$$dest");
            if ($port != "ALL") fwrite ($fwf, " $port");
            fwrite ($fwf, " ! -y -j $action");
            if ($log == "YES") fwrite ($fwf, " -l");
            fwrite ($fwf, "\n");

            fwrite ($fwf, "# (#" . $ichn++ . ") Allow return input packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A input");

            fwrite ($fwf, " -p tcp");
            fwrite ($fwf, " -d \$$source \$HIPORTS -s \$$dest");
            if ($port != "ALL") fwrite ($fwf, " $port");
            fwrite ($fwf, " ! -y -j $action");
            if ($log == "YES") fwrite ($fwf, " -l");
            fwrite ($fwf, "\n");

            fwrite ($fwf, "# (#" . $ochn++ . ") Allow return output packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A output");
          break;
        }

        fwrite ($fwf, " -p tcp");
        fwrite ($fwf, " -d \$$source \$HIPORTS -s \$$dest");
        if ($port != "ALL") fwrite ($fwf, " $port");
        fwrite ($fwf, " ! -y -j $action");
        if ($log == "YES") fwrite ($fwf, " -l");
        fwrite ($fwf, "\n");
      }

      #
      # insert a reverse rule for ACCEPT (udp) packets.
      #
      if (($action == "ACCEPT") and (($proto == "udp") or ($proto == "ALL"))) {

        # This is very ugly too.
        switch ($direction) {
          case "IN":
            fwrite ($fwf, "# (#" . $ichn++ . ") Allow return input packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A input");

            fwrite ($fwf, " -p udp");
            fwrite ($fwf, " -d \$$source \$HIPORTS -s \$$dest");
            if ($port != "ALL") fwrite ($fwf, " $port");
            fwrite ($fwf, " -j $action");
            if ($log == "YES") fwrite ($fwf, " -l");
            fwrite ($fwf, "\n");

            fwrite ($fwf, "# (#" . $ochn++ . ") Allow return output packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A output");
          break;

          case "OUT":
            fwrite ($fwf, "# (#" . $ichn++ . ") Allow return input packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A input");
          break;

          case "FWD":
            fwrite ($fwf, "# (#" . $fchn++ . ") Allow return fwd packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A forward");
          break;

          case "IN/FWD":
            fwrite ($fwf, "# (#" . $fchn++ . ") Allow return fwd packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A forward");

            fwrite ($fwf, " -p udp");
            fwrite ($fwf, " -d \$$source \$HIPORTS -s \$$dest");
            if ($port != "ALL") fwrite ($fwf, " $port");
            fwrite ($fwf, " -j $action");
            if ($log == "YES") fwrite ($fwf, " -l");
            fwrite ($fwf, "\n");

            fwrite ($fwf, "# (#" . $ichn++ . ") Allow return input packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A input");

            fwrite ($fwf, " -p udp");
            fwrite ($fwf, " -d \$$source \$HIPORTS -s \$$dest");
            if ($port != "ALL") fwrite ($fwf, " $port");
            fwrite ($fwf, " -j $action");
            if ($log == "YES") fwrite ($fwf, " -l");
            fwrite ($fwf, "\n");

            fwrite ($fwf, "# (#" . $ochn++ . ") Allow return output packets.\n");
            fwrite ($fwf, "\$IPCHAINS -A output");
          break;
        }

        fwrite ($fwf, " -p udp");
        fwrite ($fwf, " -d \$$source \$HIPORTS -s \$$dest");
        if ($port != "ALL") fwrite ($fwf, " $port");
        fwrite ($fwf, " -j $action");
        if ($log == "YES") fwrite ($fwf, " -l");
        fwrite ($fwf, "\n");
      }
    }
  }
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "# Default Rules\n");
  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Default forward policy: deny and log.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# (#" . $fchn++ . ")\n");
  fwrite ($fwf, "\$IPCHAINS -A forward -j DENY -l\n");
  fwrite ($fwf, "\$IPCHAINS -P forward DENY\n");
  fwrite ($fwf, "\n");

  if ($outrules == 0) {
    fwrite ($fwf, "#\n");
    fwrite ($fwf, "# Default output policy: accept.\n");
    fwrite ($fwf, "#\n");
    fwrite ($fwf, "\$IPCHAINS -A output -j ACCEPT\n");
    fwrite ($fwf, "\$IPCHAINS -P output ACCEPT\n");
    fwrite ($fwf, "\n");
  } else {
    fwrite ($fwf, "#\n");
    fwrite ($fwf, "# Default output policy: deny and log.\n");
    fwrite ($fwf, "#\n");
    fwrite ($fwf, "# (#" . $ochn++ . ")\n");
    fwrite ($fwf, "\$IPCHAINS -A output -j DENY -l\n");
    fwrite ($fwf, "\$IPCHAINS -P output DENY\n");
    fwrite ($fwf, "\n");
  }

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Default input policy: deny and log.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# (#" . $ichn++ . ")\n");
  fwrite ($fwf, "\$IPCHAINS -A input -j DENY -l\n");
  fwrite ($fwf, "\$IPCHAINS -P input DENY\n");

  fclose($fwf);
}

##############################################################################
##############################################################################
###
### New script for iptables ... beware !!!
###
##############################################################################
##############################################################################

function gen_script_iptables() {

  #
  # Note that this function is a very experimental first cut, and doesn't
  # work and hasn't been tested and if you use it then you're stupid.
  #

  global $CONF;

  $protocols = read_protocols();
  $services = read_services();
  $netobjects = read_objects();
  $rules = read_rulesets();
  $portfwds = read_portfwds();
  $redirectors = read_redirectors();

  $ichn = 1;
  $fchn = 1;
  $ochn = 1;
  $outrules = 0;

  #
  # Generate the file.
  #
  $fwf = fopen("$CONF/firewall", "w+");
  chmod( "$CONF/firewall", 0755 );

  fwrite ($fwf, "#!/bin/sh\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Firewall script generated by the PHP Firewall Generator (phpfwgen).\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# PHP Firewall Generator by Del (del@babel.com.au).  Original coding ideas\n");
  fwrite ($fwf, "# from IPCHAINS-FIREWALL V1.6.2m by Ian Hall-Beyer\n");
  fwrite ($fwf, "# and various others.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# This script is automatically generated.  Edit at your own peril.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# phpfwgen (C) Del under the GPL.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# This script is _not_ copyright to anyone, since it was generated by\n");
  fwrite ($fwf, "# software.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "# Initialise Firewall Configuration\n");
  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "IPTABLES='/sbin/iptables'\n");
  fwrite ($fwf, "IPMASQADM='/usr/sbin/ipmasqadm'\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Network object definitions.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "HIPORTS='1024:65535'\n");
  fwrite ($fwf, "\n");

  if (is_array($netobjects)) {
    reset($netobjects);
    while ( list( $key, $val ) = each( $netobjects ) ) {
      $exploded = explode (" ", $val);
      fwrite ($fwf, "$key='$exploded[0]/$exploded[1]'\n");
    }
  }
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Turn on forwarding and dynamic addressing.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "echo 1 > /proc/sys/net/ipv4/ip_forward\n");
  fwrite ($fwf, "echo 1 > /proc/sys/net/ipv4/ip_dynaddr\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Turn on anti-spoofing.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "for file in /proc/sys/net/ipv4/conf/*/rp_filter; do\n");
  fwrite ($fwf, "  echo 1 > \$file\n");
  fwrite ($fwf, "done\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Set IP Masquerade timeouts to sensible defaults.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "\$IPTABLES -M -S 7200 10 60\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Flush all rulesets.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "\$IPTABLES -F INPUT\n");
  fwrite ($fwf, "\$IPTABLES -F OUTPUT\n");
  fwrite ($fwf, "\$IPTABLES -F FORWARD\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Allow loopback interface\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# (#" . $ichn++ . ")\n");
  fwrite ($fwf, "\$IPTABLES -A INPUT -i lo -s \$ALL -d \$ALL -j ACCEPT\n");
  fwrite ($fwf, "# (#" . $ochn++ . ")\n");
  fwrite ($fwf, "\$IPTABLES -A OUTPUT -o lo -s \$ALL -d \$ALL -j ACCEPT\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "# Begin Rulesets\n");
  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "# Forwarding / Masquerading\n");
  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Turn off masquerading within masqueraded networks.\n");
  fwrite ($fwf, "#\n");

  if (is_array($netobjects)) {
    $netobjects_copy = $netobjects;
    reset($netobjects);
    while ( list($key,$val) = each($netobjects) ) {
      $exploded = explode (" ", $val);
      if ($exploded[2] == "YES") {
        #
        # Turn off masquerading between masq'd networks as well ...
        #
        reset($netobjects_copy);
        while ( list($keyc,$valc) = each($netobjects_copy) ) {
          $explodedc = explode (" ", $valc);
          if ($explodedc[2] == "YES") {
            fwrite ($fwf, "# (#" . $fchn++ . ")\n");
            fwrite ($fwf, "\$IPTABLES -A FORWARD -s \$$key -d \$$keyc -j ACCEPT\n");
          }
        }
      }
    }
  }

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Turn off masquerading of the external interface.\n");
  fwrite ($fwf, "#\n");
  if (isset ($netobjects["GATEWAYNET"])) {
    fwrite ($fwf, "# (#" . $fchn++ . ")\n");
    fwrite ($fwf, "\$IPTABLES -A FORWARD -s \$GATEWAYNET -d \$ALL -j ACCEPT\n");
    fwrite ($fwf, "\n");
  }

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Masquerade all other networks.\n");
  fwrite ($fwf, "#\n");

  if (is_array($netobjects)) {
    reset($netobjects);
    while ( list($key,$val) = each($netobjects) ) {
      $exploded = explode (" ", $val);
      if ($exploded[2] == "YES") {
        fwrite ($fwf, "# (#" . $fchn++ . ")\n");
        #
        # Note that this no longer works in iptables ... it's done by
        # ipnatctl instead, and I haven't figured out how that works
        # yet.  -Del-
        #
        fwrite ($fwf, "\$IPTABLES -A FORWARD -s \$$key -d \$ALL -j MASQ\n");
      }
    }
  }
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Port Forwarding.\n");
  fwrite ($fwf, "#\n");

  if (is_array($portfwds)) {
    reset($portfwds);
    while ( list($key,$rule) = each($portfwds) ) {
      $rule = chop($rule);
      fwrite ($fwf, "#\n# Rule number $key [ $rule ]\n#\n");
      list ($proto, $source, $sport, $dest, $dport) = preg_split ("/[\s]+/", $rule);
      fwrite ($fwf, "\$IPMASQADM portfw -a -P $proto");
      list ($saddr, $mask, $masq) = preg_split ("/[\s]+/", $netobjects[$source]);
      list ($daddr, $mask, $masq) = preg_split ("/[\s]+/", $netobjects[$dest]);
      fwrite ($fwf, " -L $saddr $sport -R $daddr $dport\n");
    }
  }
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Redirectors.\n");
  fwrite ($fwf, "#\n");

  if (is_array($redirectors)) {
    reset($redirectors);
    while ( list($key,$rule) = each($redirectors) ) {
      $rule = chop($rule);
      fwrite ($fwf, "#\n# Rule number $key [ $rule ]\n#\n");
      list ($source, $dest, $proto, $sport, $dport) = preg_split ("/[\s]+/", $rule);
      fwrite ($fwf, "# (#" . $ichn++ . ")\n");
      #
      # Note that this no longer works in iptables ... it's done by
      # ipnatctl instead, and I haven't figured out how that works
      # yet.  -Del-
      #
      fwrite ($fwf, "\$IPTABLES -A INPUT -p $proto");
      fwrite ($fwf, " -s \$$source -d \$$dest $sport -j REDIRECT $dport\n");
    }
  }
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "# Input/Forward/Output Firewall Rules\n");
  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "\n");

  if (is_array($rules)) {
    reset($rules);
    while ( list($key,$rule) = each($rules) ) {
      $rule = chop($rule);
      list ($source, $dest, $proto, $port, $action, $log, $direction)
        = preg_split ("/[\s]+/", $rule);
      # Do this for compatibility with previous versions of phpfwgen which
      # didn't use the direction flag ...
      if (empty($direction)) {
        $direction = "IN/FWD";
        $rule = "$rule $direction";
      }

      fwrite ($fwf, "#\n# Rule number $key [ $rule ]\n#\n");
      #
      # Yes, I know this is ugly ...
      #
      switch ($direction) {
        case "IN":
          fwrite ($fwf, "# (#" . $ichn++ . ")\n");
          fwrite ($fwf, "\$IPTABLES -A INPUT");
        break;

        case "OUT":
          fwrite ($fwf, "# (#" . $ochn++ . ")\n");
          fwrite ($fwf, "\$IPTABLES -A OUTPUT");
          $outrules = 1;
        break;

        case "FWD":
          fwrite ($fwf, "# (#" . $fchn++ . ")\n");
          fwrite ($fwf, "\$IPTABLES -A FORWARD");
        break;

        case "IN/FWD":
          fwrite ($fwf, "# (#" . $fchn++ . ")\n");
          fwrite ($fwf, "\$IPTABLES -A FORWARD");
          if ($proto != "ALL") fwrite ($fwf, " -p $proto");
          if ($proto == "icmp") {
            fwrite ($fwf, " -s \$$source");
            if ($port != "ALL") fwrite ($fwf, " $port");
            fwrite ($fwf, " -d \$$dest");
          } else {
            fwrite ($fwf, " -s \$$source -d \$$dest");
            if ($port != "ALL") fwrite ($fwf, " --dport $port");
          }
          fwrite ($fwf, " -j $action");
          if ($log == "YES") fwrite ($fwf, " -l");
          fwrite ($fwf, "\n");

          fwrite ($fwf, "# (#" . $ichn++ . ")\n");
          fwrite ($fwf, "\$IPTABLES -A INPUT");
        break;

      }

      if ($proto != "ALL") fwrite ($fwf, " -p $proto");
      if ($proto == "icmp") {
        fwrite ($fwf, " -s \$$source");
        if ($port != "ALL") fwrite ($fwf, " $port");
        fwrite ($fwf, " -d \$$dest");
      } else {
        fwrite ($fwf, " -s \$$source -d \$$dest");
        if ($port != "ALL") fwrite ($fwf, " --dport $port");
      }
      fwrite ($fwf, " -j $action");
      if ($log == "YES") fwrite ($fwf, " -l");
      fwrite ($fwf, "\n");

      #
      # insert a reverse rule for ACCEPT (tcp) packets.
      #
      if (($action == "ACCEPT") and (($proto == "tcp") or ($proto == "ALL"))) {

        # This is even more ugly.
        switch ($direction) {
          case "IN":
            fwrite ($fwf, "# (#" . $ichn++ . ") Allow return INPUT packets.\n");
            fwrite ($fwf, "\$IPTABLES -A INPUT");

            fwrite ($fwf, " -p tcp");
            fwrite ($fwf, " -d \$$source \$HIPORTS -s \$$dest");
            if ($port != "ALL") fwrite ($fwf, " --sport $port");
            fwrite ($fwf, " ! -y -j $action");
            if ($log == "YES") fwrite ($fwf, " -l");
            fwrite ($fwf, "\n");

            fwrite ($fwf, "# (#" . $ochn++ . ") Allow return OUTPUT packets.\n");
            fwrite ($fwf, "\$IPTABLES -A OUTPUT");
          break;

          case "OUT":
            fwrite ($fwf, "# (#" . $ichn++ . ") Allow return INPUT packets.\n");
            fwrite ($fwf, "\$IPTABLES -A INPUT");
          break;

          case "FWD":
            fwrite ($fwf, "# (#" . $fchn++ . ") Allow return fwd packets.\n");
            fwrite ($fwf, "\$IPTABLES -A FORWARD");
          break;

          case "IN/FWD":
            fwrite ($fwf, "# (#" . $fchn++ . ") Allow return fwd packets.\n");
            fwrite ($fwf, "\$IPTABLES -A FORWARD");

            fwrite ($fwf, " -p tcp");
            fwrite ($fwf, " -d \$$source --dport \$HIPORTS -s \$$dest");
            if ($port != "ALL") fwrite ($fwf, " --sport $port");
            fwrite ($fwf, " ! -y -j $action");
            if ($log == "YES") fwrite ($fwf, " -l");
            fwrite ($fwf, "\n");

            fwrite ($fwf, "# (#" . $ichn++ . ") Allow return INPUT packets.\n");
            fwrite ($fwf, "\$IPTABLES -A INPUT");

            fwrite ($fwf, " -p tcp");
            fwrite ($fwf, " -d \$$source --dport \$HIPORTS -s \$$dest");
            if ($port != "ALL") fwrite ($fwf, " --sport $port");
            fwrite ($fwf, " ! -y -j $action");
            if ($log == "YES") fwrite ($fwf, " -l");
            fwrite ($fwf, "\n");

            fwrite ($fwf, "# (#" . $ochn++ . ") Allow return OUTPUT packets.\n");
            fwrite ($fwf, "\$IPTABLES -A OUTPUT");
          break;
        }

        fwrite ($fwf, " -p tcp");
        fwrite ($fwf, " -d \$$source --dport \$HIPORTS -s \$$dest");
        if ($port != "ALL") fwrite ($fwf, " --sport $port");
        fwrite ($fwf, " ! -y -j $action");
        if ($log == "YES") fwrite ($fwf, " -l");
        fwrite ($fwf, "\n");
      }

      #
      # insert a reverse rule for ACCEPT (udp) packets.
      #
      if (($action == "ACCEPT") and (($proto == "udp") or ($proto == "ALL"))) {

        # This is very ugly too.
        switch ($direction) {
          case "IN":
            fwrite ($fwf, "# (#" . $ichn++ . ") Allow return INPUT packets.\n");
            fwrite ($fwf, "\$IPTABLES -A INPUT");

            fwrite ($fwf, " -p udp");
            fwrite ($fwf, " -d \$$source --dport \$HIPORTS -s \$$dest");
            if ($port != "ALL") fwrite ($fwf, " --sport $port");
            fwrite ($fwf, " -j $action");
            if ($log == "YES") fwrite ($fwf, " -l");
            fwrite ($fwf, "\n");

            fwrite ($fwf, "# (#" . $ochn++ . ") Allow return OUTPUT packets.\n");
            fwrite ($fwf, "\$IPTABLES -A OUTPUT");
          break;

          case "OUT":
            fwrite ($fwf, "# (#" . $ichn++ . ") Allow return INPUT packets.\n");
            fwrite ($fwf, "\$IPTABLES -A INPUT");
          break;

          case "FWD":
            fwrite ($fwf, "# (#" . $fchn++ . ") Allow return fwd packets.\n");
            fwrite ($fwf, "\$IPTABLES -A FORWARD");
          break;

          case "IN/FWD":
            fwrite ($fwf, "# (#" . $fchn++ . ") Allow return fwd packets.\n");
            fwrite ($fwf, "\$IPTABLES -A FORWARD");

            fwrite ($fwf, " -p udp");
            fwrite ($fwf, " -d \$$source --dport \$HIPORTS -s \$$dest");
            if ($port != "ALL") fwrite ($fwf, " --sport $port");
            fwrite ($fwf, " -j $action");
            if ($log == "YES") fwrite ($fwf, " -l");
            fwrite ($fwf, "\n");

            fwrite ($fwf, "# (#" . $ichn++ . ") Allow return INPUT packets.\n");
            fwrite ($fwf, "\$IPTABLES -A INPUT");

            fwrite ($fwf, " -p udp");
            fwrite ($fwf, " -d \$$source --dport \$HIPORTS -s \$$dest");
            if ($port != "ALL") fwrite ($fwf, " --sport $port");
            fwrite ($fwf, " -j $action");
            if ($log == "YES") fwrite ($fwf, " -l");
            fwrite ($fwf, "\n");

            fwrite ($fwf, "# (#" . $ochn++ . ") Allow return OUTPUT packets.\n");
            fwrite ($fwf, "\$IPTABLES -A OUTPUT");
          break;
        }

        fwrite ($fwf, " -p udp");
        fwrite ($fwf, " -d \$$source --dport \$HIPORTS -s \$$dest");
        if ($port != "ALL") fwrite ($fwf, " --sport $port");
        fwrite ($fwf, " -j $action");
        if ($log == "YES") fwrite ($fwf, " -l");
        fwrite ($fwf, "\n");
      }
    }
  }
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "# Default Rules\n");
  fwrite ($fwf, "#############################################################################\n");
  fwrite ($fwf, "\n");

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Default FORWARD policy: deny and log.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# (#" . $fchn++ . ")\n");
  fwrite ($fwf, "\$IPTABLES -A FORWARD -j DROP -l\n");
  fwrite ($fwf, "\$IPTABLES -P FORWARD DROP\n");
  fwrite ($fwf, "\n");

  if ($outrules == 0) {
    fwrite ($fwf, "#\n");
    fwrite ($fwf, "# Default OUTPUT policy: accept.\n");
    fwrite ($fwf, "#\n");
    fwrite ($fwf, "\$IPTABLES -A OUTPUT -j ACCEPT\n");
    fwrite ($fwf, "\$IPTABLES -P OUTPUT ACCEPT\n");
    fwrite ($fwf, "\n");
  } else {
    fwrite ($fwf, "#\n");
    fwrite ($fwf, "# Default OUTPUT policy: deny and log.\n");
    fwrite ($fwf, "#\n");
    fwrite ($fwf, "# (#" . $ochn++ . ")\n");
    fwrite ($fwf, "\$IPTABLES -A OUTPUT -j DROP -l\n");
    fwrite ($fwf, "\$IPTABLES -P OUTPUT DROP\n");
    fwrite ($fwf, "\n");
  }

  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# Default INPUT policy: deny and log.\n");
  fwrite ($fwf, "#\n");
  fwrite ($fwf, "# (#" . $ichn++ . ")\n");
  fwrite ($fwf, "\$IPTABLES -A INPUT -j DROP -l\n");
  fwrite ($fwf, "\$IPTABLES -P INPUT DROP\n");

  fclose($fwf);
}
?>
